import validator from 'validator';
import { CONSTRAINTS_FIELDS } from '../../../initializers/constants.js';
import { exists, isArray, isDateValid } from '../misc.js';
import { isActivityPubUrlValid, isBaseActivityValid, setValidAttributedTo } from './misc.js';
import { isHostValid } from '../servers.js';
import { peertubeTruncate } from '../../core-utils.js';
function isActorEndpointsObjectValid(endpointObject) {
    if (endpointObject === null || endpointObject === void 0 ? void 0 : endpointObject.sharedInbox) {
        return isActivityPubUrlValid(endpointObject.sharedInbox);
    }
    return true;
}
function isActorPublicKeyObjectValid(publicKeyObject) {
    return isActivityPubUrlValid(publicKeyObject.id) &&
        isActivityPubUrlValid(publicKeyObject.owner) &&
        isActorPublicKeyValid(publicKeyObject.publicKeyPem);
}
function isActorTypeValid(type) {
    return type === 'Person' || type === 'Application' || type === 'Group' || type === 'Service' || type === 'Organization';
}
function isActorPublicKeyValid(publicKey) {
    return exists(publicKey) &&
        typeof publicKey === 'string' &&
        publicKey.startsWith('-----BEGIN PUBLIC KEY-----') &&
        publicKey.includes('-----END PUBLIC KEY-----') &&
        validator.default.isLength(publicKey, CONSTRAINTS_FIELDS.ACTORS.PUBLIC_KEY);
}
const actorNameAlphabet = '[ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789\\-_.:]';
const actorNameRegExp = new RegExp(`^${actorNameAlphabet}+$`);
function isActorPreferredUsernameValid(preferredUsername) {
    return exists(preferredUsername) && validator.default.matches(preferredUsername, actorNameRegExp);
}
function isActorPrivateKeyValid(privateKey) {
    return exists(privateKey) &&
        typeof privateKey === 'string' &&
        (privateKey.startsWith('-----BEGIN RSA PRIVATE KEY-----') || privateKey.startsWith('-----BEGIN PRIVATE KEY-----')) &&
        (privateKey.includes('-----END RSA PRIVATE KEY-----') || privateKey.includes('-----END PRIVATE KEY-----')) &&
        validator.default.isLength(privateKey, CONSTRAINTS_FIELDS.ACTORS.PRIVATE_KEY);
}
function isActorFollowingCountValid(value) {
    return exists(value) && validator.default.isInt('' + value, { min: 0 });
}
function isActorFollowersCountValid(value) {
    return exists(value) && validator.default.isInt('' + value, { min: 0 });
}
function isActorDeleteActivityValid(activity) {
    return isBaseActivityValid(activity, 'Delete');
}
function sanitizeAndCheckActorObject(actor) {
    if (!isActorTypeValid(actor.type))
        return false;
    normalizeActor(actor);
    return exists(actor) &&
        isActivityPubUrlValid(actor.id) &&
        isActivityPubUrlValid(actor.inbox) &&
        isActorPreferredUsernameValid(actor.preferredUsername) &&
        isActivityPubUrlValid(actor.url) &&
        isActorPublicKeyObjectValid(actor.publicKey) &&
        isActorEndpointsObjectValid(actor.endpoints) &&
        (!actor.outbox || isActivityPubUrlValid(actor.outbox)) &&
        (!actor.following || isActivityPubUrlValid(actor.following)) &&
        (!actor.followers || isActivityPubUrlValid(actor.followers)) &&
        setValidAttributedTo(actor) &&
        setValidDescription(actor) &&
        (actor.type !== 'Group' || actor.attributedTo.length !== 0);
}
function normalizeActor(actor) {
    if (!actor)
        return;
    if (!actor.url) {
        actor.url = actor.id;
    }
    else if (typeof actor.url !== 'string') {
        actor.url = actor.url.href || actor.url.url;
    }
    if (!isDateValid(actor.published))
        actor.published = undefined;
    if (actor.summary && typeof actor.summary === 'string') {
        actor.summary = peertubeTruncate(actor.summary, { length: CONSTRAINTS_FIELDS.USERS.DESCRIPTION.max });
        if (actor.summary.length < CONSTRAINTS_FIELDS.USERS.DESCRIPTION.min) {
            actor.summary = null;
        }
    }
}
function isValidActorHandle(handle) {
    if (!exists(handle))
        return false;
    const parts = handle.split('@');
    if (parts.length !== 2)
        return false;
    return isHostValid(parts[1]);
}
function areValidActorHandles(handles) {
    return isArray(handles) && handles.every(h => isValidActorHandle(h));
}
function setValidDescription(obj) {
    if (!obj.summary)
        obj.summary = null;
    return true;
}
export { normalizeActor, actorNameAlphabet, areValidActorHandles, isActorEndpointsObjectValid, isActorPublicKeyObjectValid, isActorTypeValid, isActorPublicKeyValid, isActorPreferredUsernameValid, isActorPrivateKeyValid, isActorFollowingCountValid, isActorFollowersCountValid, isActorDeleteActivityValid, sanitizeAndCheckActorObject, isValidActorHandle };
//# sourceMappingURL=actor.js.map