import httpSignature from '@peertube/http-signature';
import { createWriteStream } from 'fs';
import { remove } from 'fs-extra/esm';
import got from 'got';
import { HttpProxyAgent, HttpsProxyAgent } from 'hpagent';
import { ACTIVITY_PUB, BINARY_CONTENT_TYPES, PEERTUBE_VERSION, REQUEST_TIMEOUTS, WEBSERVER } from '../initializers/constants.js';
import { pipelinePromise } from './core-utils.js';
import { logger, loggerTagsFactory } from './logger.js';
import { getProxy, isProxyEnabled } from './proxy.js';
const lTags = loggerTagsFactory('request');
const peertubeGot = got.extend(Object.assign(Object.assign({}, getAgent()), { headers: {
        'user-agent': getUserAgent()
    }, handlers: [
        (options, next) => {
            var _a;
            const promiseOrStream = next(options);
            const bodyKBLimit = (_a = options.context) === null || _a === void 0 ? void 0 : _a.bodyKBLimit;
            if (!bodyKBLimit)
                throw new Error('No KB limit for this request');
            const bodyLimit = bodyKBLimit * 1000;
            promiseOrStream.on('downloadProgress', progress => {
                if (progress.transferred > bodyLimit && progress.percent !== 1) {
                    const message = `Exceeded the download limit of ${bodyLimit} B`;
                    logger.warn(message, lTags());
                    if (promiseOrStream.cancel) {
                        promiseOrStream.cancel();
                        return;
                    }
                    promiseOrStream.destroy();
                }
            });
            return promiseOrStream;
        }
    ], hooks: {
        beforeRequest: [
            options => {
                const headers = options.headers || {};
                headers['host'] = buildUrl(options.url).host;
            },
            options => {
                var _a, _b;
                const httpSignatureOptions = (_a = options.context) === null || _a === void 0 ? void 0 : _a.httpSignature;
                if (httpSignatureOptions) {
                    const method = (_b = options.method) !== null && _b !== void 0 ? _b : 'GET';
                    const path = buildUrl(options.url).pathname;
                    if (!method || !path) {
                        throw new Error(`Cannot sign request without method (${method}) or path (${path}) ${options}`);
                    }
                    httpSignature.signRequest({
                        getHeader: function (header) {
                            const value = options.headers[header.toLowerCase()];
                            if (!value)
                                logger.warn('Unknown header requested by http-signature.', { headers: options.headers, header });
                            return value;
                        },
                        setHeader: function (header, value) {
                            options.headers[header] = value;
                        },
                        method,
                        path
                    }, httpSignatureOptions);
                }
            }
        ],
        beforeRetry: [
            (error, retryCount) => {
                logger.debug('Retrying request to %s.', error.request.requestUrl, Object.assign({ retryCount, error: buildRequestError(error) }, lTags()));
            }
        ]
    } }));
function doRequest(url, options = {}) {
    const gotOptions = buildGotOptions(options);
    return peertubeGot(url, gotOptions)
        .catch(err => { throw buildRequestError(err); });
}
function doJSONRequest(url, options = {}) {
    const gotOptions = buildGotOptions(options);
    return peertubeGot(url, Object.assign(Object.assign({}, gotOptions), { responseType: 'json' }))
        .catch(err => { throw buildRequestError(err); });
}
async function doRequestAndSaveToFile(url, destPath, options = {}) {
    var _a;
    const gotOptions = buildGotOptions(Object.assign(Object.assign({}, options), { timeout: (_a = options.timeout) !== null && _a !== void 0 ? _a : REQUEST_TIMEOUTS.FILE }));
    const outFile = createWriteStream(destPath);
    try {
        await pipelinePromise(peertubeGot.stream(url, Object.assign(Object.assign({}, gotOptions), { isStream: true })), outFile);
    }
    catch (err) {
        remove(destPath)
            .catch(err => logger.error('Cannot remove %s after request failure.', destPath, Object.assign({ err }, lTags())));
        throw buildRequestError(err);
    }
}
function getAgent() {
    if (!isProxyEnabled())
        return {};
    const proxy = getProxy();
    logger.info('Using proxy %s.', proxy, lTags());
    const proxyAgentOptions = {
        keepAlive: true,
        keepAliveMsecs: 1000,
        maxSockets: 256,
        maxFreeSockets: 256,
        scheduling: 'lifo',
        proxy
    };
    return {
        agent: {
            http: new HttpProxyAgent(proxyAgentOptions),
            https: new HttpsProxyAgent(proxyAgentOptions)
        }
    };
}
function getUserAgent() {
    return `PeerTube/${PEERTUBE_VERSION} (+${WEBSERVER.URL})`;
}
function isBinaryResponse(result) {
    return BINARY_CONTENT_TYPES.has(result.headers['content-type']);
}
export { doRequest, doJSONRequest, doRequestAndSaveToFile, isBinaryResponse, getAgent, peertubeGot };
function buildGotOptions(options) {
    var _a;
    const { activityPub, bodyKBLimit = 1000 } = options;
    const context = { bodyKBLimit, httpSignature: options.httpSignature };
    let headers = options.headers || {};
    if (!headers.date) {
        headers = Object.assign(Object.assign({}, headers), { date: new Date().toUTCString() });
    }
    if (activityPub && !headers.accept) {
        headers = Object.assign(Object.assign({}, headers), { accept: ACTIVITY_PUB.ACCEPT_HEADER });
    }
    return {
        method: options.method,
        dnsCache: true,
        timeout: {
            request: (_a = options.timeout) !== null && _a !== void 0 ? _a : REQUEST_TIMEOUTS.DEFAULT
        },
        json: options.json,
        searchParams: options.searchParams,
        followRedirect: options.followRedirect,
        retry: {
            limit: 2
        },
        headers,
        context
    };
}
function buildRequestError(error) {
    const newError = new Error(error.message);
    newError.name = error.name;
    newError.stack = error.stack;
    if (error.response) {
        newError.responseBody = error.response.body;
        newError.responseHeaders = error.response.headers;
        newError.statusCode = error.response.statusCode;
    }
    if (error.options) {
        newError.requestHeaders = error.options.headers;
    }
    return newError;
}
function buildUrl(url) {
    if (typeof url === 'string') {
        return new URL(url);
    }
    return url;
}
//# sourceMappingURL=requests.js.map