import Bluebird from 'bluebird';
import { computeOutputFPS } from '../../../../helpers/ffmpeg/index.js';
import { logger } from '../../../../helpers/logger.js';
import { CONFIG } from '../../../../initializers/config.js';
import { DEFAULT_AUDIO_RESOLUTION, VIDEO_TRANSCODING_FPS } from '../../../../initializers/constants.js';
import { JobQueue } from '../../../job-queue/index.js';
import { Hooks } from '../../../plugins/hooks.js';
import { VideoPathManager } from '../../../video-path-manager.js';
import { VideoJobInfoModel } from '../../../../models/video/video-job-info.js';
import { ffprobePromise, getVideoStreamDimensionsInfo, getVideoStreamFPS, hasAudioStream, isAudioFile } from '@peertube/peertube-ffmpeg';
import { getTranscodingJobPriority } from '../../transcoding-priority.js';
import { canDoQuickTranscode } from '../../transcoding-quick-transcode.js';
import { buildOriginalFileResolution, computeResolutionsToTranscode } from '../../transcoding-resolutions.js';
import { AbstractJobBuilder } from './abstract-job-builder.js';
export class TranscodingJobQueueBuilder extends AbstractJobBuilder {
    async createOptimizeOrMergeAudioJobs(options) {
        const { video, videoFile, isNewVideo, user, videoFileAlreadyLocked } = options;
        let mergeOrOptimizePayload;
        let nextTranscodingSequentialJobPayloads = [];
        const mutexReleaser = videoFileAlreadyLocked
            ? () => { }
            : await VideoPathManager.Instance.lockFiles(video.uuid);
        try {
            await video.reload();
            await videoFile.reload();
            await VideoPathManager.Instance.makeAvailableVideoFile(videoFile.withVideoOrPlaylist(video), async (videoFilePath) => {
                const probe = await ffprobePromise(videoFilePath);
                const { resolution } = await getVideoStreamDimensionsInfo(videoFilePath, probe);
                const hasAudio = await hasAudioStream(videoFilePath, probe);
                const quickTranscode = await canDoQuickTranscode(videoFilePath, probe);
                const inputFPS = videoFile.isAudio()
                    ? VIDEO_TRANSCODING_FPS.AUDIO_MERGE
                    : await getVideoStreamFPS(videoFilePath, probe);
                const maxResolution = await isAudioFile(videoFilePath, probe)
                    ? DEFAULT_AUDIO_RESOLUTION
                    : buildOriginalFileResolution(resolution);
                if (CONFIG.TRANSCODING.HLS.ENABLED === true) {
                    nextTranscodingSequentialJobPayloads.push([
                        this.buildHLSJobPayload({
                            deleteWebVideoFiles: CONFIG.TRANSCODING.WEB_VIDEOS.ENABLED === false,
                            copyCodecs: !quickTranscode,
                            resolution: maxResolution,
                            fps: computeOutputFPS({ inputFPS, resolution: maxResolution }),
                            videoUUID: video.uuid,
                            isNewVideo
                        })
                    ]);
                }
                const lowerResolutionJobPayloads = await this.buildLowerResolutionJobPayloads({
                    video,
                    inputVideoResolution: maxResolution,
                    inputVideoFPS: inputFPS,
                    hasAudio,
                    isNewVideo
                });
                nextTranscodingSequentialJobPayloads = [...nextTranscodingSequentialJobPayloads, ...lowerResolutionJobPayloads];
                const hasChildren = nextTranscodingSequentialJobPayloads.length !== 0;
                mergeOrOptimizePayload = videoFile.isAudio()
                    ? this.buildMergeAudioPayload({ videoUUID: video.uuid, isNewVideo, hasChildren })
                    : this.buildOptimizePayload({ videoUUID: video.uuid, isNewVideo, quickTranscode, hasChildren });
            });
        }
        finally {
            mutexReleaser();
        }
        const nextTranscodingSequentialJobs = await Bluebird.mapSeries(nextTranscodingSequentialJobPayloads, payloads => {
            return Bluebird.mapSeries(payloads, payload => {
                return this.buildTranscodingJob({ payload, user });
            });
        });
        const transcodingJobBuilderJob = {
            type: 'transcoding-job-builder',
            payload: {
                videoUUID: video.uuid,
                sequentialJobs: nextTranscodingSequentialJobs
            }
        };
        const mergeOrOptimizeJob = await this.buildTranscodingJob({ payload: mergeOrOptimizePayload, user });
        await JobQueue.Instance.createSequentialJobFlow(...[mergeOrOptimizeJob, transcodingJobBuilderJob]);
        await VideoJobInfoModel.increaseOrCreate(video.uuid, 'pendingTranscode');
    }
    async createTranscodingJobs(options) {
        const { video, transcodingType, resolutions, isNewVideo } = options;
        const maxResolution = Math.max(...resolutions);
        const childrenResolutions = resolutions.filter(r => r !== maxResolution);
        logger.info('Manually creating transcoding jobs for %s.', transcodingType, { childrenResolutions, maxResolution });
        const { fps: inputFPS } = await video.probeMaxQualityFile();
        const children = childrenResolutions.map(resolution => {
            const fps = computeOutputFPS({ inputFPS, resolution });
            if (transcodingType === 'hls') {
                return this.buildHLSJobPayload({ videoUUID: video.uuid, resolution, fps, isNewVideo });
            }
            if (transcodingType === 'webtorrent' || transcodingType === 'web-video') {
                return this.buildWebVideoJobPayload({ videoUUID: video.uuid, resolution, fps, isNewVideo });
            }
            throw new Error('Unknown transcoding type');
        });
        const fps = computeOutputFPS({ inputFPS, resolution: maxResolution });
        const parent = transcodingType === 'hls'
            ? this.buildHLSJobPayload({ videoUUID: video.uuid, resolution: maxResolution, fps, isNewVideo })
            : this.buildWebVideoJobPayload({ videoUUID: video.uuid, resolution: maxResolution, fps, isNewVideo });
        await this.createTranscodingJobsWithChildren({ videoUUID: video.uuid, parent, children, user: null });
    }
    async createTranscodingJobsWithChildren(options) {
        const { videoUUID, parent, children, user } = options;
        const parentJob = await this.buildTranscodingJob({ payload: parent, user });
        const childrenJobs = await Bluebird.mapSeries(children, c => this.buildTranscodingJob({ payload: c, user }));
        await JobQueue.Instance.createJobWithChildren(parentJob, childrenJobs);
        await VideoJobInfoModel.increaseOrCreate(videoUUID, 'pendingTranscode', 1 + children.length);
    }
    async buildTranscodingJob(options) {
        const { user, payload } = options;
        return {
            type: 'video-transcoding',
            priority: await getTranscodingJobPriority({ user, type: 'vod', fallback: undefined }),
            payload
        };
    }
    async buildLowerResolutionJobPayloads(options) {
        const { video, inputVideoResolution, inputVideoFPS, isNewVideo, hasAudio } = options;
        const resolutionsEnabled = await Hooks.wrapObject(computeResolutionsToTranscode({ input: inputVideoResolution, type: 'vod', includeInput: false, strictLower: true, hasAudio }), 'filter:transcoding.auto.resolutions-to-transcode.result', options);
        const sequentialPayloads = [];
        for (const resolution of resolutionsEnabled) {
            const fps = computeOutputFPS({ inputFPS: inputVideoFPS, resolution });
            if (CONFIG.TRANSCODING.WEB_VIDEOS.ENABLED) {
                const payloads = [
                    this.buildWebVideoJobPayload({
                        videoUUID: video.uuid,
                        resolution,
                        fps,
                        isNewVideo
                    })
                ];
                if (CONFIG.TRANSCODING.HLS.ENABLED) {
                    payloads.push(this.buildHLSJobPayload({
                        videoUUID: video.uuid,
                        resolution,
                        fps,
                        isNewVideo,
                        copyCodecs: true
                    }));
                }
                sequentialPayloads.push(payloads);
            }
            else if (CONFIG.TRANSCODING.HLS.ENABLED) {
                sequentialPayloads.push([
                    this.buildHLSJobPayload({
                        videoUUID: video.uuid,
                        resolution,
                        fps,
                        copyCodecs: false,
                        isNewVideo
                    })
                ]);
            }
        }
        return sequentialPayloads;
    }
    buildHLSJobPayload(options) {
        const { videoUUID, resolution, fps, isNewVideo, deleteWebVideoFiles = false, copyCodecs = false } = options;
        return {
            type: 'new-resolution-to-hls',
            videoUUID,
            resolution,
            fps,
            copyCodecs,
            isNewVideo,
            deleteWebVideoFiles
        };
    }
    buildWebVideoJobPayload(options) {
        const { videoUUID, resolution, fps, isNewVideo } = options;
        return {
            type: 'new-resolution-to-web-video',
            videoUUID,
            isNewVideo,
            resolution,
            fps
        };
    }
    buildMergeAudioPayload(options) {
        const { videoUUID, isNewVideo, hasChildren } = options;
        return {
            type: 'merge-audio-to-web-video',
            resolution: DEFAULT_AUDIO_RESOLUTION,
            fps: VIDEO_TRANSCODING_FPS.AUDIO_MERGE,
            videoUUID,
            isNewVideo,
            hasChildren
        };
    }
    buildOptimizePayload(options) {
        const { videoUUID, quickTranscode, isNewVideo, hasChildren } = options;
        return {
            type: 'optimize-to-web-video',
            videoUUID,
            isNewVideo,
            hasChildren,
            quickTranscode
        };
    }
}
//# sourceMappingURL=transcoding-job-queue-builder.js.map