import { VideoPrivacy, VideoState } from '@peertube/peertube-models';
import { CONFIG } from '../initializers/config.js';
import { VideoJobInfoModel } from '../models/video/video-job-info.js';
import { JobQueue } from './job-queue/job-queue.js';
import { moveFilesIfPrivacyChanged } from './video-privacy.js';
export async function buildMoveJob(options) {
    const { video, previousVideoState, isNewVideo = true, type } = options;
    await VideoJobInfoModel.increaseOrCreate(video.uuid, 'pendingMove');
    return {
        type,
        payload: {
            videoUUID: video.uuid,
            isNewVideo,
            previousVideoState
        }
    };
}
export function buildStoryboardJobIfNeeded(options) {
    const { video, federate } = options;
    if (CONFIG.STORYBOARDS.ENABLED) {
        return {
            type: 'generate-video-storyboard',
            payload: {
                videoUUID: video.uuid,
                federate
            }
        };
    }
    if (federate === true) {
        return {
            type: 'federate-video',
            payload: {
                videoUUID: video.uuid,
                isNewVideoForFederation: false
            }
        };
    }
    return undefined;
}
export async function addVideoJobsAfterCreation(options) {
    const { video, videoFile } = options;
    const jobs = [
        {
            type: 'manage-video-torrent',
            payload: {
                videoId: video.id,
                videoFileId: videoFile.id,
                action: 'create'
            }
        },
        buildStoryboardJobIfNeeded({ video, federate: false }),
        {
            type: 'notify',
            payload: {
                action: 'new-video',
                videoUUID: video.uuid
            }
        },
        {
            type: 'federate-video',
            payload: {
                videoUUID: video.uuid,
                isNewVideoForFederation: true
            }
        }
    ];
    if (video.state === VideoState.TO_MOVE_TO_EXTERNAL_STORAGE) {
        jobs.push(await buildMoveJob({ video, previousVideoState: undefined, type: 'move-to-object-storage' }));
    }
    if (video.state === VideoState.TO_TRANSCODE) {
        jobs.push({
            type: 'transcoding-job-builder',
            payload: {
                videoUUID: video.uuid,
                optimizeJob: {
                    isNewVideo: true
                }
            }
        });
    }
    return JobQueue.Instance.createSequentialJobFlow(...jobs);
}
export async function addVideoJobsAfterUpdate(options) {
    const { video, nameChanged, oldPrivacy, isNewVideoForFederation } = options;
    const jobs = [];
    const filePathChanged = await moveFilesIfPrivacyChanged(video, oldPrivacy);
    if (!video.isLive && (nameChanged || filePathChanged)) {
        for (const file of (video.VideoFiles || [])) {
            const payload = { action: 'update-metadata', videoId: video.id, videoFileId: file.id };
            jobs.push({ type: 'manage-video-torrent', payload });
        }
        const hls = video.getHLSPlaylist();
        for (const file of ((hls === null || hls === void 0 ? void 0 : hls.VideoFiles) || [])) {
            const payload = { action: 'update-metadata', streamingPlaylistId: hls.id, videoFileId: file.id };
            jobs.push({ type: 'manage-video-torrent', payload });
        }
    }
    jobs.push({
        type: 'federate-video',
        payload: {
            videoUUID: video.uuid,
            isNewVideoForFederation
        }
    });
    const wasConfidentialVideo = new Set([
        VideoPrivacy.PRIVATE,
        VideoPrivacy.UNLISTED,
        VideoPrivacy.INTERNAL
    ]).has(oldPrivacy);
    if (wasConfidentialVideo) {
        jobs.push({
            type: 'notify',
            payload: {
                action: 'new-video',
                videoUUID: video.uuid
            }
        });
    }
    return JobQueue.Instance.createSequentialJobFlow(...jobs);
}
//# sourceMappingURL=video-jobs.js.map