import { pick } from '@peertube/peertube-core-utils';
import { VideoResolution } from '@peertube/peertube-models';
import { readFile, writeFile } from 'fs/promises';
import { dirname } from 'path';
import { FFmpegCommandWrapper } from './ffmpeg-command-wrapper.js';
import { ffprobePromise, getVideoStreamDimensionsInfo } from './ffprobe.js';
import { presetCopy, presetOnlyAudio, presetVOD } from './shared/presets.js';
export class FFmpegVOD {
    constructor(options) {
        this.ended = false;
        this.commandWrapper = new FFmpegCommandWrapper(options);
    }
    async transcode(options) {
        const builders = {
            'quick-transcode': this.buildQuickTranscodeCommand.bind(this),
            'hls': this.buildHLSVODCommand.bind(this),
            'hls-from-ts': this.buildHLSVODFromTSCommand.bind(this),
            'merge-audio': this.buildAudioMergeCommand.bind(this),
            'video': this.buildWebVideoCommand.bind(this)
        };
        this.commandWrapper.debugLog('Will run transcode.', { options });
        const command = this.commandWrapper.buildCommand(options.inputPath)
            .output(options.outputPath);
        await builders[options.type](options);
        command.on('start', () => {
            setTimeout(() => {
                if (options.inputFileMutexReleaser) {
                    options.inputFileMutexReleaser();
                }
            }, 1000);
        });
        await this.commandWrapper.runCommand();
        await this.fixHLSPlaylistIfNeeded(options);
        this.ended = true;
    }
    isEnded() {
        return this.ended;
    }
    async buildWebVideoCommand(options) {
        const { resolution, fps, inputPath } = options;
        if (resolution === VideoResolution.H_NOVIDEO) {
            presetOnlyAudio(this.commandWrapper);
            return;
        }
        let scaleFilterValue;
        if (resolution !== undefined) {
            const probe = await ffprobePromise(inputPath);
            const videoStreamInfo = await getVideoStreamDimensionsInfo(inputPath, probe);
            scaleFilterValue = (videoStreamInfo === null || videoStreamInfo === void 0 ? void 0 : videoStreamInfo.isPortraitMode) === true
                ? `w=${resolution}:h=-2`
                : `w=-2:h=${resolution}`;
        }
        await presetVOD({
            commandWrapper: this.commandWrapper,
            resolution,
            input: inputPath,
            canCopyAudio: true,
            canCopyVideo: true,
            fps,
            scaleFilterValue
        });
    }
    buildQuickTranscodeCommand(_options) {
        const command = this.commandWrapper.getCommand();
        presetCopy(this.commandWrapper);
        command.outputOption('-map_metadata -1')
            .outputOption('-movflags faststart');
    }
    async buildAudioMergeCommand(options) {
        const command = this.commandWrapper.getCommand();
        command.loop(undefined);
        await presetVOD(Object.assign(Object.assign({}, pick(options, ['resolution'])), { commandWrapper: this.commandWrapper, input: options.audioPath, canCopyAudio: true, canCopyVideo: true, fps: options.fps, scaleFilterValue: this.getMergeAudioScaleFilterValue() }));
        command.outputOption('-preset:v veryfast');
        command.input(options.audioPath)
            .outputOption('-tune stillimage')
            .outputOption('-shortest');
    }
    getMergeAudioScaleFilterValue() {
        return 'trunc(iw/2)*2:trunc(ih/2)*2';
    }
    async buildHLSVODCommand(options) {
        const command = this.commandWrapper.getCommand();
        const videoPath = this.getHLSVideoPath(options);
        if (options.copyCodecs)
            presetCopy(this.commandWrapper);
        else if (options.resolution === VideoResolution.H_NOVIDEO)
            presetOnlyAudio(this.commandWrapper);
        else
            await this.buildWebVideoCommand(options);
        this.addCommonHLSVODCommandOptions(command, videoPath);
    }
    buildHLSVODFromTSCommand(options) {
        const command = this.commandWrapper.getCommand();
        const videoPath = this.getHLSVideoPath(options);
        command.outputOption('-c copy');
        if (options.isAAC) {
            command.outputOption('-bsf:a aac_adtstoasc');
        }
        this.addCommonHLSVODCommandOptions(command, videoPath);
    }
    addCommonHLSVODCommandOptions(command, outputPath) {
        return command.outputOption('-hls_time 4')
            .outputOption('-hls_list_size 0')
            .outputOption('-hls_playlist_type vod')
            .outputOption('-hls_segment_filename ' + outputPath)
            .outputOption('-hls_segment_type fmp4')
            .outputOption('-f hls')
            .outputOption('-hls_flags single_file');
    }
    async fixHLSPlaylistIfNeeded(options) {
        if (options.type !== 'hls' && options.type !== 'hls-from-ts')
            return;
        const fileContent = await readFile(options.outputPath);
        const videoFileName = options.hlsPlaylist.videoFilename;
        const videoFilePath = this.getHLSVideoPath(options);
        const newContent = fileContent.toString()
            .replace(`#EXT-X-MAP:URI="${videoFilePath}",`, `#EXT-X-MAP:URI="${videoFileName}",`);
        await writeFile(options.outputPath, newContent);
    }
    getHLSVideoPath(options) {
        return `${dirname(options.outputPath)}/${options.hlsPlaylist.videoFilename}`;
    }
}
//# sourceMappingURL=ffmpeg-vod.js.map